﻿// analysis_cmdline_args.cpp
#include "analysis_cmdline_args.h"
#include <algorithm>
#include <cassert>
#include <iterator>

using namespace std;

namespace cmd_args {

    static OptionInfo help_info {};
    static OptionInfo version_info {};


    // コマンドライン引数を std::vector<std::string> に格納して返す
    vector<string> argv_to_vector(int argc, char** argv)
    {
        vector<string> vec {};
        for (int i {0}; i < argc; ++i) {
            vec.push_back(argv[i]);
        }
        return vec;
    }

    // ヘルプオプションを登録する
    void regist_help(const OptionInfo& info)
    {
        assert(info.short_name != "" || info.long_name != "");
        assert(info.callback_func);

        help_info = info;
    }

    // バージョン情報オプションを登録する
    void regist_version(const OptionInfo& info)
    {
        assert(info.short_name != "" || info.long_name != "");
        assert(info.callback_func);

        version_info = info;
    }

    // コマンドライン引数を解析する
    AnalysisCmdlineArgsResult analysis_cmdline_args(const vector<string>& cmdline_args, std::size_t option_start_index, const vector<OptionInfo>& options)
    {
        // コマンドライン引数の個数チェック
        if (cmdline_args.size() < 2) {
            return AnalysisCmdlineArgsResult::error_args_not_enough;
        }

        // ヘルプ
        if (cmdline_args.at(1) == help_info.short_name || cmdline_args.at(1) == help_info.long_name) {
            help_info.callback_func("");
            return AnalysisCmdlineArgsResult::correct_exit;
        }

        // バージョン情報
        if (cmdline_args.at(1) == version_info.short_name || cmdline_args.at(1) == version_info.long_name) {
            version_info.callback_func("");
            return AnalysisCmdlineArgsResult::correct_exit;
        }

        // オプションを探す
        for (size_t i {option_start_index}; i < cmdline_args.size(); ++i) {
            auto it_opt = find_if(
                cbegin(options),
                cend(options),
                [arg = cmdline_args.at(i)](const OptionInfo& opt) {
                    return arg == opt.short_name || arg == opt.long_name;
                }
            );

            if (it_opt == cend(options)) {
                // 該当するオプションがない
                return AnalysisCmdlineArgsResult::error_invalid_option;
            }
            else {
                // 発見したオプションに応じた登録関数をコールバック
                if (it_opt->has_param) {
                    ++i;
                    if (i < cmdline_args.size()) {
                        it_opt->callback_func(cmdline_args.at(i));
                    }
                    else {
                        return AnalysisCmdlineArgsResult::error_invalid_parameter;
                    }
                }
                else {
                    it_opt->callback_func("");
                }
            }
        }

        return AnalysisCmdlineArgsResult::correct;
    }
}